import pandas as pd
from ..Types import (
    VARCHAR,
    STR,
    TEXT,
    TIMESTAMP,
    DATETIME64,
    SUPER,
    INT,
    BIGINT,
    BOOLEAN,
    BOOL,
    MAX_VARCHAR_LENGTH,
    target_type_is_numeric,
)


class AdapterSourceTarget:
    def __init__(self, struct):
        self.struct = struct

    def convert_types(self, df):
        for c in self.struct.columns:
            case_target_type = {
                VARCHAR: STR,
                TEXT: STR,
                TIMESTAMP: DATETIME64,
                SUPER: STR,
                INT: pd.Int64Dtype(),
                BIGINT: pd.Int64Dtype(),
                BOOLEAN: BOOL,
            }

            if c["target_type"] in case_target_type.keys():
                if c["source_name"] not in df.columns:
                    pass
                else:
                    df[c["source_name"]] = df[c["source_name"]].astype(
                        case_target_type[c["target_type"]]
                    )
                    if case_target_type[c["target_type"]] == STR:
                        df[c["source_name"]].replace("None", "", inplace=True)

        return df

    def transform_data(self, df):
        for c in self.struct.columns:
            if (c["target_type"] == VARCHAR or c["target_type"] == TEXT) and c[
                "source_name"
            ] in df.columns:
                df[c["source_name"]] = df[c["source_name"]].str[:MAX_VARCHAR_LENGTH]
        return df

    def equalize_number_columns(self, df):
        df_columns = df.columns.tolist()
        struct_columns = [c["source_name"] for c in self.struct.columns]

        for dfc in df_columns:
            if dfc not in struct_columns and dfc != "Op":
                df = df.drop(columns=[dfc])

        for sc in struct_columns:
            if sc not in df_columns:

                def find_target_type(sc):
                    for c in self.struct.columns:
                        if c["source_name"] == sc:
                            return c["target_type"]

                target_type = find_target_type(sc)

                if target_type == VARCHAR or target_type == TEXT:
                    df[sc] = ""
                elif target_type_is_numeric(target_type):
                    df[sc] = None
                else:
                    df[sc] = None

        df = df[struct_columns]

        return df
